// 
// JTLT - a templating system for the java language
// Copyright (C) 2001-2003 Indrek Mandre <indrek@mare.ee>
// All rights reserved.
// 
// See the file "AUTHORS" for the name of all copyright holders.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program in the file "COPYING-LESSER"; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
// 
// For more information and latest updates please see
// http://www.mare.ee/indrek/jtlt/
// 

package ee.mare.indrek.jtlt;

import ee.mare.indrek.jtlt.macros.*;
import java.util.*;
import java.io.*;

/** An example file based alias database loader. This
  * system is not througly-developed. */
public class FilebasedAliasDatabase {
  /** A mockup implementation of filesystem based alias database. */
  public static Map loadFilebasedAliasDatabase (String path) throws IOException
  {
    FileReader fr = new FileReader (path);
    Map ret = loadFilebasedAliasDatabase (fr);
    fr.close ();
    return ret;
  }

  /** A mockup implementation of filesystem based alias database. */
  public static Map loadFilebasedAliasDatabase (Reader in) throws IOException
  {
    Map ret = new HashMap ();
    Map all = new HashMap ();
    Map deflt = new HashMap ();
    ret.put ("all", all);
    ret.put ("DEFAULT", deflt);

    BufferedReader bin = new BufferedReader (in);
    String line;

    while ( (line = bin.readLine()) != null ) {
        if ( line.startsWith("#") ) continue; // # marks comments

        // Alias file line format: <lang> : <options> : <key> : <content>
        // Where <options> is <key>=<value>&<key>=<value>&.. where in
        // <key> and <value> '&' and '=' are not allowed.
        // In content the slashes must be escaped, \r and \r are resolved

        StringTokenizer tok = new StringTokenizer (line, ":");
        if ( tok.countTokens() < 4 ) continue;

        String lang = (String)tok.nextToken();
        String opt = ((String)tok.nextToken()).trim();
        String key = (String)tok.nextToken();
        String content = (String)tok.nextToken("").substring(1);
        content = aliasContentDecode (content);

        Map a2c = (Map)ret.get(lang);
        if ( a2c == null ) {
            a2c = new HashMap ();
            ret.put (lang, a2c);
        }

        AliasKey ak = new AliasKey ();
        ak.key = key;
        ak.lang = lang;
        ak.opt = opt;
        ak.content = content;

        all.put (lang + ":" + key, ak);

        a2c.put (key, ak);
    }

    // Resolve the deflt aliases
    Iterator lit = ret.entrySet().iterator();
    while ( lit.hasNext () ) {
        Map.Entry lentry = (Map.Entry)lit.next();
        String lname = (String)lentry.getKey();
        if ( lname.equals ("all") || lname.equals ("DEFAULT") ) continue;
        Map keys = (Map)lentry.getValue();
        Iterator kit = keys.entrySet().iterator();
        while ( kit.hasNext() ) {
            Map.Entry kentry = (Map.Entry)kit.next();
            AliasKey ak = (AliasKey)kentry.getValue();
            ak.deflt = (AliasKey)deflt.get(ak.key);
        }
    }

    return ret;
  }

  public static String aliasContentDecode (String in)
  {
    StringBuffer out = new StringBuffer ();
    int n = 0;

    while (n < in.length() ) {
        char c = in.charAt(n);
        if ( c == '\\' && n < in.length() - 1 ) {
            char nc = in.charAt(n+1);
            switch (nc) {
              case '\\':
                out.append ("\\");
                break;
              case 'r':
                out.append ("\r");
                break;
              case 'n':
                out.append ("\n");
                break;
              default:  // we should never get here
                out.append (nc);
                break;
            }
            n++;
        } else {
          out.append (c);
        }
        n++;
    }

    return out.toString ();
  }

  public static String aliasContentEncode (String in)
  {
    StringBuffer out = new StringBuffer ();
    int n = 0;

    while ( n < in.length() ) {
        char c = in.charAt(n);
        switch (c) {
          case '\\':
            out.append ("\\\\");
            break;
          case '\r':
            out.append ("\\r");
            break;
          case '\n':
            out.append ("\\n");
            break;
          default:
            out.append (c);
            break;
        }
        n++;
    }

    return out.toString ();
  }
}

