//
// JTLT - a templating system for the java language
// Copyright (C) 2001-2003 Indrek Mandre <indrek@mare.ee>
// All rights reserved.
//
// See the file "AUTHORS" for the name of all copyright holders.
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program in the file "COPYING-LESSER"; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
//
// For more information and latest updates please see
// http://www.mare.ee/indrek/jtlt/
//

package ee.mare.indrek.jtlt;

import java.io.*;
import java.util.*;

/** Html specialized template parser and generator. */
public class HtmlTemplateGenerator extends TemplateGenerator {
  /** Create a new generator using the default HtmlTemplateContext. */
  public HtmlTemplateGenerator (Reader in) throws IOException, TemplateException
  {
    this(in, default_ctx);
  }

  /** Create a new generator using the default HtmlTemplateContext. */
  public HtmlTemplateGenerator (String path) throws IOException, TemplateException
  {
    this(path, default_ctx);
  }

  /** Create a new generator. */
  public HtmlTemplateGenerator (String path, HtmlTemplateContext ctx) throws IOException, TemplateException
  {
    super(path, ctx);
  }

  /** Create a new generator. */
  public HtmlTemplateGenerator (Reader in, HtmlTemplateContext ctx) throws IOException, TemplateException
  {
    super(in, ctx);
  }

  /** Create a new HtmlTemplate with given language code used for localization. */
  public HtmlTemplate createHtmlTemplate (String lang)
  {
    HtmlTemplate ret = new HtmlTemplate (this);
    Map alias_db = ((HtmlTemplateContext)getTemplateContext()).getAliasDatabase();
    if ( alias_db != null ) {
        ret.addParam ("alias_db_all", alias_db.get("all"));
        ret.addParam ("language", lang);
    }
    return ret;
  }

  /** Create a new HtmlTemplate instance. The localization info will be "DEFAULT". */
  public HtmlTemplate createHtmlTemplate ()
  {
    return createHtmlTemplate ("DEFAULT");
  }

  public Template createTemplate ()
  {
    return createHtmlTemplate ();
  }

  private static HtmlTemplateContext default_ctx = new HtmlTemplateContext ();
}

