//
// JTLT - a templating system for the java language
// Copyright (C) 2001-2003 Indrek Mandre <indrek@mare.ee>
// All rights reserved.
//
// See the file "AUTHORS" for the name of all copyright holders.
//
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this program in the file "COPYING-LESSER"; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
//
// For more information and latest updates please see
// http://www.mare.ee/indrek/jtlt/
//

package ee.mare.indrek.jtlt;

import java.io.*;
import java.util.*;

/** Read in the template and try to beautify the input to
  * to get nicer looking output from templates.
  * That is whenever there is a single line that starts with
  * SUB and ends with tag end or starts with END then
  * from end and start all whitespace is trimmed, including
  * newlines. But if line starts with sub or end, the white
  * space before is removed. */
public class TemplateBeautifyReader extends CharArrayReader {
  public TemplateBeautifyReader (Reader reader, TemplateContext ctx) throws IOException
  {
    super(new char[] { });

    CharArrayWriter out = new CharArrayWriter ();
    BufferedReader in = new BufferedReader (reader);

    String stag_sub = ctx.getStartTag() + "SUB:";
    String stag_end = ctx.getStartTag() + "END:";
    String etag = ctx.getEndTag();

    String line;
    while ( ( line = in.readLine() ) != null ) {
        String test = line.trim();
        if ( test.startsWith(stag_sub) && test.endsWith (etag) ) {
            out.write (test);
            continue;
        }

        if ( test.startsWith(stag_end) && test.endsWith (etag) ) {
            out.write (test);
            continue;
        }

        if ( test.startsWith(stag_sub) || test.startsWith(stag_end) ) {
            out.write (test);
            out.write ("\n");
            continue;
        }

        out.write(line);
        out.write("\n");
    }

    buf = out.toCharArray();
    count = out.size();
    markedPos = pos = 0;
  }
}

