// 
// JTLT - a templating system for the java language
// Copyright (C) 2001-2003 Indrek Mandre <indrek@mare.ee>
// All rights reserved.
// 
// See the file "AUTHORS" for the name of all copyright holders.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program in the file "COPYING-LESSER"; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
// 
// For more information and latest updates please see
// http://www.mare.ee/indrek/jtlt/
// 

package ee.mare.indrek.jtlt;

import java.util.*;
import java.io.*;

/** Class that represents a block inside template. It is used by
  * the macro extensions for quering macro parameters.
  * @see TemplateMacro
  */
class TemplateBlock implements TemplateParams {
  TemplateBlock (Template t, TemplateGenerator g, TemplateBlock p)
  {
    tlt = t;
    generator = g;
    parent = p;
  }

  TemplateGenerator generator;
  private Template tlt;
  TemplateBlock parent;

  void add_sub (TemplateBlock s, boolean to_back)
  {
    List l = (List)tlt.get_content (this, s.generator);
    if ( l == null ) {
        l = new LinkedList ();
        tlt.set_content (this, s.generator, l);
    }
    if ( to_back )
        l.add(s);
    else
        l.add(0, s);
  }

  void append (Writer add) throws IOException
  {
    do_append (add, generator.getParts());
  }

  private void do_append (Writer add, List list) throws IOException
  {
    Iterator i = list.iterator();
    while ( i.hasNext() ) {
        Object o = i.next();
        if ( o instanceof TemplateGenerator ) {
            List l = (List)tlt.get_content (this, o);
            if ( l != null ) {
                Iterator j = l.iterator ();
                while ( j.hasNext() ) {
                    TemplateBlock c = (TemplateBlock)j.next();
                    c.append (add);
                }
            }
        } else if ( o instanceof String ) {
            add.write ((String)o);
        } else if (o instanceof MacroArgs ) {
            MacroArgs ma = (MacroArgs)o;

            Writer w;
            if ( ma.macro.isRecursive() )
                w = new StringWriter ();
            else
                w = add;

            if ( ma.args instanceof String ) {
                w.write (ma.macro.process (this, ma.name, (String)ma.args));
            } else {
                StringWriter sw = new StringWriter ();
                do_append (sw, (List)ma.args);
                w.write (ma.macro.process (this, ma.name, sw.toString()));
            }

            if ( ma.macro.isRecursive () ) {
                try {
                    do_append (add, generator.parse (((StringWriter)w).toString(), 0, false));
                } catch (TemplateException e) {
                    generator.getTemplateContext().complain ("While recursively parsin macro output", e);
                }
            }
        } else {
          // this should never happen unless bug in template code
          System.err.println ("Invalid part of type " + o.getClass().getName());
        }
    }
  }

  public Object findParam (String ident, boolean search_parent)
  {
    Object m = tlt.get_content (this, ident);
    if ( m != null ) return m;
    if ( search_parent && parent != null ) return parent.findParam (ident, search_parent);
    return null;
  }

  public TemplateContext getTemplateContext ()
  {
    return generator.getTemplateContext ();
  }

  void set_param (String identifier, Object val)
  {
    tlt.set_content (this, identifier, val);
  }
}

