// 
// JTLT - a templating system for the java language
// Copyright (C) 2001-2003 Indrek Mandre <indrek@mare.ee>
// All rights reserved.
// 
// See the file "AUTHORS" for the name of all copyright holders.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program in the file "COPYING-LESSER"; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
// 
// For more information and latest updates please see
// http://www.mare.ee/indrek/jtlt/
// 

package ee.mare.indrek.jtlt;

class MacroArgs {
  TemplateMacro macro;
  String name;
  Object args;  //< either String or List
}

/** Macros can be used to extend the template files syntax.
  * Macros must be registered at the TemplateContext you are
  * using. Whenever a macro with given name is found the
  * process method is called. Macros also allow you to use
  * parameters that are set at individual template classes.
  * @see TemplateContext#registerMacro
  */
public interface TemplateMacro {
  /** Get the name of the macro. Macro is called whenever a name
    * is used inside the template. */
  String getName ();

  /** This method is called with given block context whenever a macro
    * with registered name is found inside the tempalate. The
    * macros are resolved immediately only if they are parsetime, otherwise
    * on every Template.toString the data structures are parsed and
    * macros expanded. If macro is parsetime the ctx is taken from the generator.
    * You can add parameters into the template using the Template.addParam,
    * for parsetime macros you'd have to extend the TemplateGenerator.
    * @return the string value to be replaced into the template
    * @see TemplateParams#findParam
    * @see Template#addParam
    */
  public String process (TemplateParams ctx, String name, String args);

  /** Returns whether a macro is parsetime or not. Being parsetime means here 
    * that the macro does not depend on template instance specified
    * parameters and can be resolved at parsing time. But resolving macros
    * only at parsetime is faster than doing it on every created template.
    * Not all instances of macro usage may be parsed at parsetime tho.
    * For example when the result of recursive macro contains it.*/
  public boolean isParsetime ();

  /** Whether to recursively parse the output of this the macro. If the macro is used
    * at parsetime the result can contain additional sub blocks. */
  public boolean isRecursive ();
}

