// 
// JTLT - a templating system for the java language
// Copyright (C) 2001-2003 Indrek Mandre <indrek@mare.ee>
// All rights reserved.
// 
// See the file "AUTHORS" for the name of all copyright holders.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program in the file "COPYING-LESSER"; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
// 
// For more information and latest updates please see
// http://www.mare.ee/indrek/jtlt/
// 

package ee.mare.indrek.jtlt.macros;

import ee.mare.indrek.jtlt.*;
import java.util.*;
import java.io.*;

/** A macro (INCLUDE) for including files into a template. You should be careful
  * with using this macro as splitting a template into multiple files
  * can create a nightmare of manageability. Still sometimes it can be
  * put into good use. The included file is also parsed for all macros
  * and also sub-blocks. */
public class IncludeMacro extends MacroAdapter {
  private List path = new LinkedList();

  /** A search path where from to search for files, different paths separated
    * by either colon or semicolon. */
  public IncludeMacro (String search_path)
  {
    this (search_path, "INCLUDE");
  }

  public IncludeMacro (String search_path, String name)
  {
    super (name);
    StringTokenizer st = new StringTokenizer(search_path, File.pathSeparator);
    while ( st.hasMoreTokens() ) {
         path.add (st.nextToken());
     }
  }

  public String process (TemplateParams ctx, String name, String args)
  {
    try {
        File f = new File (args);
        if ( !f.isAbsolute () ) {
            f = null;
            Iterator it = path.iterator ();
            while ( it.hasNext () ) {
                String p = (String)it.next();
                f = new File (p + File.separator + args);
                if ( f.exists () ) break;
            }
        }

        if ( f == null || !f.exists () ) {
            ctx.getTemplateContext().complain ("INCLUDE: given file [" + args + "] not found!");
            return "";
        }

        Reader in = new FileReader (f);
        StringBuffer buf = new StringBuffer ();

        char[] b = new char[4096];
        int read = 0;
        while ( (read = in.read(b)) > 0)
            buf.append (b, 0, read);

        in.close ();
        return buf.toString();
    } catch (Exception e) {
        ctx.getTemplateContext().complain("Unable to include given file [" + args + "]", e);
        return "";
    }
  }

  public boolean isParsetime () { return true; }
  public boolean isRecursive () { return true; }
}


