// 
// JTLT - a templating system for the java language
// Copyright (C) 2001-2003 Indrek Mandre <indrek@mare.ee>
// All rights reserved.
// 
// See the file "AUTHORS" for the name of all copyright holders.
// 
// This program is free software; you can redistribute it and/or modify it
// under the terms of the GNU Lesser General Public License as published
// by the Free Software Foundation; either version 2.1 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public
// License along with this program in the file "COPYING-LESSER"; if not,
// write to the Free Software Foundation, Inc., 59 Temple Place,
// Suite 330, Boston, MA  02111-1307  USA
// 
// For more information and latest updates please see
// http://www.mare.ee/indrek/jtlt/
// 

package ee.mare.indrek.jtlt.macros;

import ee.mare.indrek.jtlt.*;
import java.util.*;
import java.io.*;

/** A macro for producing separators, like commas, for easier list creation. */
public class SepMacro extends MacroAdapter {
  private String separator;
  private String default_name;

  /** SepMacro returns a SUB with contents of separator inside it.
    * The sub will be named as macro argument or if no argument given
    * then as default_name. */
  public SepMacro (String macro_name, String default_name, String separator)
  {
    super(macro_name);
    this.default_name = default_name;
    this.separator = separator;
  }

  public String process (TemplateParams params, String name, String args)
  {
    if ( args.equals("") )
        args = default_name;

    TemplateContext ctx = params.getTemplateContext();
    StringBuffer out = new StringBuffer ();
    out.append (ctx.getStartTag());
    out.append ("SUB:");
    out.append (args);
    out.append (ctx.getEndTag());
    out.append (separator);
    out.append (ctx.getStartTag());
    out.append ("END:");
    out.append (args);
    out.append (ctx.getEndTag());

    return out.toString();
  }

  public boolean isParsetime () { return true; }
  public boolean isRecursive () { return true; }
}


